##############################################################################
#
# Copyright (c) 2002-2017 Nexedi SA and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
##############################################################################

from erp5.component.test.testOfficeJSAppstoreMixin import \
  testOfficeJSAppstoreMixin
#from unittest import expectedFailure
#from zLOG import LOG, INFO, ERROR
import time
TEST_USER_BASE = "test_eos_user"

class TestOfficeJSSecurity(testOfficeJSAppstoreMixin):

  def getTitle(self):
    return "Test OfficeJS Security"

  def checkNewMember(self, person, website_id="letitseed"):
    self.assertEqual(repr(sorted(person.getCareerRoleList())), repr(["author"]))
    assignment, = person.searchFolder(portal_type="Assignment", validation_state="open") # not "opened"!
    self.assertEqual(
      repr(sorted(assignment.getCategoryList())),
      repr(["role/author"]))

  def _newMemberNoTicNoCheck(self,
                 tag="user_creation_in_progress",
                 first_name="John_test",
                 last_name="Smith_test",
                 reference=TEST_USER_BASE,
                 email="john.smith@test.com",
                 special_id=None):
    """
    Add new member using script for the website sign-in
    """
    if special_id is not None:
      reference += "%s" % special_id
    else:
      reference += "%s" % time.time()
    # XXX
    user_list = self.portal.portal_catalog(portal_type="Person", reference=reference)
    self.assertEqual(len(user_list), 0)
    user = self.portal.person_module.newContent(
      portal_type="Person",
      first_name=first_name,
      last_name=last_name,
      default_email_coordinate_text=email,
    )
    user.validate()
    assignment = user.newContent(portal_type="Assignment", title=reference)
    assignment.setCategoryList(["role/author"])
    assignment.open()
    user.setCareerRoleList(["author"])
    return user

  def _newMemberAndTic(self, *args, **kwargs):
    user = self._newMemberNoTicNoCheck(*args, **kwargs)
    self.tic()
    self.checkNewMember(user)
    return user

  def test_01_user_see_itself_but_noone_else(self):
    person = self._newMemberNoTicNoCheck()
    second_user = self._newMemberNoTicNoCheck(special_id="%s_3" % time.time())
    self.tic()
    self.failUnlessUserCanViewDocument(person.getUserId(),
                                       person)
    self.failIfUserCanModifyDocument(person.getUserId(),
                                     person)
    self.failIfUserCanViewDocument(second_user.getUserId(),
                                   person)
